/*
 * $Id:$
 *
 * Copyright (C) 2004 Essence project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <Edb.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

#include "essence.h"
#include "essence_db.h"
#include "essence_db_item.h"
#include "misc.h"

static gboolean on_window_delete (GtkWidget *widget, GtkWidget *event,
                                  gpointer data)
{
    gtk_main_quit ();

    return FALSE;
}

static void on_action_new (GtkAction *action, Essence *e)
{
}

/* creates a file filter for EDB files */
static GtkFileFilter *get_edb_filter ()
{
	GtkFileFilter *f = gtk_file_filter_new ();

	gtk_file_filter_set_name (f, "EDB files");
	gtk_file_filter_add_pattern (f, "*.db");

	return f;
}

static void on_action_open (GtkAction *action, Essence *e)
{
	GtkWidget *dialog;
	GtkFileFilter *filter = get_edb_filter ();
	char *file;
	int n;

	dialog = gtk_file_chooser_dialog_new ("Open EDB", e->window,
	                                      GTK_FILE_CHOOSER_ACTION_OPEN,
	                                      GTK_STOCK_OPEN,
	                                      GTK_RESPONSE_ACCEPT,
	                                      GTK_STOCK_CANCEL,
	                                      GTK_RESPONSE_CANCEL, NULL);

	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);

	/* now run the dialog */
	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
		file = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));

		/* load the chosen file */
		essence_load (e, file);
		g_free (file);

		/* open the notebook page we just created */
		n = gtk_notebook_get_n_pages (e->notebook);
		gtk_notebook_set_current_page (e->notebook, n - 1);
	}

	gtk_widget_destroy (dialog);
}

static void on_action_save (GtkAction *action, Essence *e)
{
	EssenceDb *edb;
	int i;

	/* get the index of the selected notebook page */
	if ((i = gtk_notebook_get_current_page (e->notebook)) == -1)
		return;

	/* use that index to find the corresponding EssenceDb */
	edb = g_slist_nth_data (e->dbs, i);
	essence_db_save (edb);

}

static void on_action_save_as (GtkAction *action, Essence *e)
{
	EssenceDb *edb;
	GtkWidget *dialog;
	GtkFileFilter *filter = get_edb_filter ();
	char *file;
	int i;

	/* get the index of the selected notebook page */
	if ((i = gtk_notebook_get_current_page (e->notebook)) == -1)
		return;

	dialog = gtk_file_chooser_dialog_new ("Open EDB", e->window,
	                                      GTK_FILE_CHOOSER_ACTION_SAVE,
	                                      GTK_STOCK_SAVE,
	                                      GTK_RESPONSE_ACCEPT,
	                                      GTK_STOCK_CANCEL,
	                                      GTK_RESPONSE_CANCEL, NULL);

	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);

	/* now run the dialog */
	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
		file = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));

		/* save the current database as the chosen file */
		edb = g_slist_nth_data (e->dbs, i);
		essence_db_save_as (edb, file);
		g_free (file);
	}

	gtk_widget_destroy (dialog);
}

static void on_action_save_all (GtkAction *action, Essence *e)
{
	GSList *l;

	for (l = e->dbs; l; l = l->next)
		essence_db_save (l->data);
}

static void on_action_close (GtkAction *action, Essence *e)
{
	EssenceDb *edb;
	GSList *l;
	int i;

	/* get the index of the selected notebook page */
	if ((i = gtk_notebook_get_current_page (e->notebook)) == -1)
		return;

	/* use that index to find the corresponding EssenceDb */
	l = g_slist_nth (e->dbs, i);
	edb = l->data;

	essence_db_free (edb);
	e->dbs = g_slist_remove_link (e->dbs, l);
}

static void on_action_quit (GtkAction *action, Essence *e)
{
	gtk_main_quit ();
}

bool essence_init (Essence *e)
{
	GtkUIManager *ui;
	GtkActionGroup *actions;
	GtkWidget *vbox, *bar;
	static GtkActionEntry entries[] = {
		{"FileMenu", NULL, "_File"},
		{"New", GTK_STOCK_NEW, "_New",
		 "<control>n", "New",
		 G_CALLBACK (on_action_new)},
		{"Open", GTK_STOCK_OPEN, "_Open...",
		 "<control>o", "Open",
		 G_CALLBACK (on_action_open)},
		{"Save", GTK_STOCK_SAVE, "_Save",
		 "<control>s", "Save",
		 G_CALLBACK (on_action_save)},
		{"SaveAs", GTK_STOCK_SAVE_AS, "Save _As...",
		 "<shift><control>s", "Save As",
		 G_CALLBACK (on_action_save_as)},
		{"SaveAll", "", "Sav_e All",
		 "", "Save All",
		 G_CALLBACK (on_action_save_all)},
		{"Close", GTK_STOCK_CLOSE, "_Close",
		 "<control>w", "Close",
		 G_CALLBACK (on_action_close)},
		{"Quit", GTK_STOCK_QUIT, "_Quit",
		 "<control>q", "Quit",
		 G_CALLBACK (on_action_quit)}
	};
	static const gchar *ui_info =
		"<ui>"
		"	<menubar name='MenuBar'>"
		"		<menu action='FileMenu'>"
		"			<menuitem action='New'/>"
		"			<menuitem action='Open'/>"
		"			<separator action='Sep1'/>"
		"			<menuitem action='Save'/>"
		"			<menuitem action='SaveAs'/>"
		"			<menuitem action='SaveAll'/>"
		"			<separator action='Sep2'/>"
		"			<menuitem action='Close'/>"
		"			<separator action='Sep3'/>"
		"			<menuitem action='Quit'/>"
		"		</menu>"
		"	</menubar>"
		"	<toolbar name='ToolBar'>"
		"		<toolitem action='New'/>"
		"		<toolitem action='Open'/>"
		"		<toolitem action='Save'/>"
		"	</toolbar>"
		"</ui>";

	assert (e);

	/* create the main window */
	e->window = GTK_WINDOW (gtk_window_new (GTK_WINDOW_TOPLEVEL));
	gtk_window_set_title (e->window, "Essence");
	gtk_window_set_default_size (e->window, 640, 480);
	gtk_window_set_position (e->window, GTK_WIN_POS_CENTER);

	g_signal_connect (G_OBJECT (e->window), "delete_event",
	                  G_CALLBACK (on_window_delete), NULL);

	/* set up the actiongroup and the UI manager */
	actions = gtk_action_group_new ("Actions");
	gtk_action_group_add_actions (actions, entries,
	                              G_N_ELEMENTS (entries), e);

	ui = gtk_ui_manager_new ();
	g_object_set_data_full (G_OBJECT (e->window), "ui-manager", ui,
	                        g_object_unref);

	gtk_ui_manager_insert_action_group (ui, actions, 0);
	gtk_ui_manager_add_ui_from_string (ui, ui_info, -1, NULL);

	vbox = gtk_vbox_new (FALSE, 0);

	/* now get the menubar and the toolbar and add them
	 * to the vbox as well.
	 */
	bar = gtk_ui_manager_get_widget (ui, "/MenuBar");
	gtk_box_pack_start (GTK_BOX (vbox), bar,
	                    FALSE, FALSE, 0);

	bar = gtk_ui_manager_get_widget (ui, "/ToolBar");
	gtk_toolbar_set_style (GTK_TOOLBAR (bar), GTK_TOOLBAR_ICONS);
	gtk_box_pack_start (GTK_BOX (vbox), bar,
	                    FALSE, FALSE, 0);

	e->notebook = GTK_NOTEBOOK (gtk_notebook_new ());
	gtk_notebook_set_tab_pos (e->notebook, GTK_POS_TOP);
	gtk_container_set_border_width (GTK_CONTAINER (e->notebook), 12);

	gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET (e->notebook),
	                    TRUE, TRUE, 6);

	/* finally add the vbox to the main window.
	 * the main window will be shown just before gtk_main() */
	gtk_container_add (GTK_CONTAINER (e->window), vbox);

	return true;
}

Essence *essence_new ()
{
	return calloc (1, sizeof (Essence));
}

void essence_free (Essence *e)
{
	EssenceDb *edb;
	assert (e);

	while (e->dbs){
		edb = e->dbs->data;
		essence_db_free (edb);
		e->dbs = g_slist_remove_link (e->dbs, e->dbs);
	}

	free (e);
}

static void on_essence_db_destroy (Essence *e, EssenceDb *edb)
{
	assert (e);
	assert (edb);

	e->dbs = g_slist_remove (e->dbs, edb);
}

bool essence_load (Essence *e, const char *file)
{
	EssenceDb *db;

	assert (e);
	assert (file);

	/* create a new EssenceDB object */
	if (!(db = essence_db_new (e->notebook, file,
	                           (GFunc) on_essence_db_destroy, e)))
		return false;

	e->dbs = g_slist_append (e->dbs, db);

	return essence_db_load (db);
}
