/*
 * $Id:$
 *
 * Copyright (C) 2004 Essence project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <Edb.h>
#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "essence.h"
#include "essence_db.h"
#include "essence_db_item.h"
#include "misc.h"

EssenceDbItem *essence_db_item_new (E_DB_File *db)
{
	EssenceDbItem *ei;

	assert (db);

	if (!(ei = calloc (1, sizeof (EssenceDbItem))))
		return NULL;

	ei->db = db;

	return ei;
}

void essence_db_item_free (EssenceDbItem *ei)
{
	assert (ei);

	free (ei);
}

void essence_db_item_key_set (EssenceDbItem *ei, const char *key)
{
	assert (ei);
	assert (key);

	/* remove the old entry first */
	if (ei->key && ei->key[0])
		essence_db_item_remove (ei);

	strlcpy (ei->key, key, sizeof (ei->key));
}

bool essence_db_item_load (EssenceDbItem *ei)
{
	char *type, *tmp;
	int i = 0;
	float f = 0;

	assert (ei);
	assert (ei->db);
	assert (ei->key);
	assert (ei->key[0]);

	ei->type = ITEM_TYPE_NONE;

	if (!(type = e_db_type_get (ei->db, ei->key)))
		return false;

	if (!strcmp (type, "str")) {
		if ((tmp = e_db_str_get (ei->db, ei->key))) {
			ei->type = ITEM_TYPE_STR;
			strlcpy (ei->data.s, tmp, sizeof (ei->data.s));
			free (tmp);
		}
	} else if (!strcmp (type, "int")) {
		if (e_db_int_get (ei->db, ei->key, &i)) {
			ei->type = ITEM_TYPE_INT;
			ei->data.i = i;
		}
	} else if (!strcmp (type, "float")) {
		if (e_db_float_get (ei->db, ei->key, &f)) {
			ei->type = ITEM_TYPE_FLOAT;
			ei->data.f = f;
		}
	}

	free (type);

	return ei->type != ITEM_TYPE_NONE;
}

GtkTreeIter essence_db_item_show (EssenceDbItem *ei,
                                  GtkListStore *store)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	const char *type[ITEM_TYPE_NUM] = {"", "Integer", "Float", "String"};
	char value[1024];

	assert (ei);
	assert (store);
	assert (ei->type != ITEM_TYPE_NONE);

	model = GTK_TREE_MODEL(store);

	switch (ei->type) {
		case ITEM_TYPE_INT:
			snprintf (value, sizeof (value), "%i", ei->data.i);
			break;
		case ITEM_TYPE_FLOAT:
			snprintf (value, sizeof (value), "%f", ei->data.f);
			break;
		case ITEM_TYPE_STR:
			strlcpy (value, ei->data.s, sizeof (value));
			break;
		default:
			break;
	}

	gtk_list_store_append (store, &iter);
	gtk_list_store_set (store, &iter,
	                    COL_KEY, ei->key,
	                    COL_TYPE, type[ei->type],
	                    COL_VALUE, value,
	                    COL_ITEM, ei, -1);

	return iter;
}

void essence_db_item_remove (EssenceDbItem *ei)
{
	assert (ei);
	assert (ei->key[0]);

	e_db_data_del (ei->db, ei->key);
}

void essence_db_item_save (EssenceDbItem *ei)
{
	assert (ei);
	assert (ei->key[0]);
	assert (ei->type);

	switch (ei->type) {
		case ITEM_TYPE_STR:
			e_db_str_set (ei->db, ei->key, ei->data.s);
			break;
		case ITEM_TYPE_INT:
			e_db_int_set (ei->db, ei->key, ei->data.i);
			break;
		case ITEM_TYPE_FLOAT:
			e_db_float_set (ei->db, ei->key, ei->data.f);
			break;
		default:
			assert (false);
	}
}
